from river.multiclass.occ import OutputCodeClassifier
from river.multiclass.ovo import OneVsOneClassifier
from river.multiclass.ovr import OneVsRestClassifier

__all__ = ["OutputCodeClassifier", "OneVsOneClassifier", "OneVsRestClassifier"]